#include <errno.h>
#include <getopt.h>
#include <net_funcs.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <utility.h>

void
print_help()
{
    printf(
        "Usage is:\nnetdog [-p port] [-f filename] [-i] <desintation> "
        "<port>\n* "
        "Port must "
        "be specified either with -p or after <desination>, but not both (Only "
        "supports TCP)\n* If "
        "-f is specified, input will be read from <filename> instead of "
        "stdin\n* If -i is specified with -f, an interactive session will be "
        "started after file send is complete\n* "
        "Desination may "
        "be either a hostname or IPv4/6\n");
}

int
main(int argc, char **argv)
{
    int32_t  opt         = 0;
    uint64_t port_num    = 0;
    char *   port        = NULL;
    char *   destination = NULL;
    char *   filename    = NULL;
    bool     inter_con   = false;
    bool     set_port    = false;
    bool     did_error   = false;

    // Check at least the 2 required args
    if (3 > argc)
    {
        goto INVALID_ARGS;
    }

    // Parse flag arguments
    while (-1 != (opt = getopt(argc, argv, "hlip:f:")))
    {
        switch (opt)
        {
            case 'p':
                set_port = true;
                port     = optarg;
                break;
            case 'f':
                filename = optarg;
                break;
            case 'i':
                inter_con = true;
                break;
            case 'h':
                print_help();
                goto EXIT;
            default:
                goto INVALID_ARGS;
        }
    }

    // Parse destination
    destination = argv[optind];

    // If port was not set with flag, and port positon is not null, get port
    if ((false == set_port) && (NULL != argv[optind + 1]))
    {
        port = argv[optind + 1];
    }

    // check for trailing "junk arguments"
    if ((true == set_port) && (NULL != argv[optind + 1]))
    {
        fprintf(stderr, "Junk arguments after desintation\n");
        goto INVALID_ARGS;
    }

    // convert number for bounds checking purposes
    if (false == convert_num(port, &port_num))
    {
        goto INVALID_ARGS;
    }

    // UINT16_MAX is largest possible port (2^16-1)
    if ((NULL == destination) || (UINT16_MAX < port_num) || (0 == port_num))
    {
        goto INVALID_ARGS;
    }

    if (NULL != filename)
    {
        printf("[+] Reading input from: %s\n", filename);
    }

    DEBUG_PRINT(("Dest:%s\nPort:%s\n", destination, port));

    did_error = client(destination, port, filename, inter_con);

    goto EXIT;

INVALID_ARGS:
    fprintf(stderr, "Invalid Arguments\n");
    print_help();
    did_error = true;
    goto EXIT;

FAILED:
    did_error = true;

EXIT:
    return did_error;
}